/*
    Scanbot Barcode Scanner SDK Flutter Plugin

    Copyright (c) 2020 doo GmbH. All rights reserved.
    https://scanbot.io/sdk
 */
package io.scanbot.barcode.sdk.flutter

import android.app.Activity
import android.util.Log
import io.flutter.plugin.common.MethodCall
import io.scanbot.barcode.sdk.flutter.utils.ObjectMapper
import io.scanbot.sdk.barcode.entity.*
import io.scanbot.sdk.ui.camera.FinderAspectRatio
import io.scanbot.sdk.ui.view.barcode.batch.configuration.BatchBarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeImageGenerationType
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeScannerAdditionalConfiguration
import io.scanbot.sdk.ui.view.barcode.configuration.BarcodeScannerConfiguration
import io.scanbot.sdk.ui.view.base.configuration.CameraOrientationMode
import java.util.*
import kotlin.collections.ArrayList


fun readBarcodeScannerConfiguration(
    call: MethodCall,
    activity: Activity
): BarcodeScannerConfiguration {
    val barcodeCameraConfiguration = BarcodeScannerConfiguration()
    val arguments = call.arguments as BarcodeScannerConfigurationJson
    val value = checkCameraOrientationParam(call, arguments)
    barcodeCameraConfiguration.setBarcodeImageGenerationType(BarcodeImageGenerationType.NONE)
    value?.let {
        barcodeCameraConfiguration.setOrientationLockMode(value)
    }
    val aspectRatio = checkFinderAspectRatioParam(call, arguments)
    aspectRatio?.let {
        barcodeCameraConfiguration.setFinderAspectRatio(it)
    }
    ObjectMapper.map(arguments, barcodeCameraConfiguration)
    (arguments.get("additionalParameters") as? Json)?.let {
        barcodeCameraConfiguration.setAdditionalDetectionParameters(getAdditionalConfig(it))
    }
    if (arguments.containsKey("engineMode")) {
        val engineMode = arguments["engineMode"] as String
        val mode = extractEngineModeFrom(engineMode)
        barcodeCameraConfiguration.setEngineMode(mode)
    }
    if (arguments.containsKey("barcodeFormats")) {
        val barcodeFormats = arguments["barcodeFormats"] as ArrayList<String>
        val nativeBarcodeFormats = ArrayList<BarcodeFormat>()
        for (formatValue in barcodeFormats) {
            nativeBarcodeFormats.add(
                BarcodeFormat.valueOf(
                    formatValue.replaceFirst(
                        "BarcodeFormat.",
                        ""
                    )
                )
            )
        }
        barcodeCameraConfiguration.setBarcodeFormatsFilter(nativeBarcodeFormats)
    }
    if (arguments.containsKey("acceptedDocumentFormats")) {
        val acceptedDocumentFormats = arguments["acceptedDocumentFormats"] as ArrayList<String>
        val acceptedBarcodeDocumentFormat = ArrayList<BarcodeDocumentFormat>()
        for (formatValue in acceptedDocumentFormats) {
            acceptedBarcodeDocumentFormat.add(
                BarcodeDocumentFormat.valueOf(
                    formatValue.replaceFirst(
                        "AcceptedDocumentFormat.",
                        ""
                    )
                )
            )
        }
        barcodeCameraConfiguration.setAcceptedDocumentFormats(acceptedBarcodeDocumentFormat)
    }
    if (arguments.containsKey("barcodeImageGenerationType")) {
        val imageGenType =
            BarcodeImageGenerationType.valueOf(arguments["barcodeImageGenerationType"] as String)
        barcodeCameraConfiguration.setBarcodeImageGenerationType(imageGenType)
    }

    if (arguments.containsKey("replaceCancelButtonWithIcon")) {
        if (arguments["replaceCancelButtonWithIcon"] == true){
            barcodeCameraConfiguration.setCancelButtonIcon(R.drawable.ic_baseline_arrow_back_24);
        }
    }
    return barcodeCameraConfiguration
}

fun readBatchBarcodeScannerConfiguration(
    call: MethodCall,
    activity: Activity
): BatchBarcodeScannerConfiguration {
    val barcodeCameraConfiguration = BatchBarcodeScannerConfiguration()
    val arguments = call.arguments as BarcodeScannerConfigurationJson
    val orientationMode = checkCameraOrientationParam(call, arguments)
    barcodeCameraConfiguration.setBarcodeImageGenerationType(BarcodeImageGenerationType.NONE)
    orientationMode?.let {
        barcodeCameraConfiguration.setOrientationLockMode(orientationMode)
    }
    val aspectRatio = checkFinderAspectRatioParam(call, arguments)
    aspectRatio?.let {
        barcodeCameraConfiguration.setFinderAspectRatio(it)
    }
    ObjectMapper.map(arguments, barcodeCameraConfiguration)
    (arguments.get("additionalParameters") as? Json)?.let {
        barcodeCameraConfiguration.setAdditionalDetectionParameters(getAdditionalConfig(it))
    }
    if (arguments.containsKey("engineMode")) {
        val engineMode = arguments["engineMode"] as String
        val mode = extractEngineModeFrom(engineMode)
        barcodeCameraConfiguration.setEngineMode(mode)
    }
    if (arguments.containsKey("barcodeFormats")) {
        val barcodeFormats = arguments["barcodeFormats"] as ArrayList<String>
        val nativeBarcodeFormats = ArrayList<BarcodeFormat>()
        for (formatValue in barcodeFormats) {
            nativeBarcodeFormats.add(
                BarcodeFormat.valueOf(
                    formatValue.replaceFirst(
                        "BarcodeFormat.",
                        ""
                    )
                )
            )
        }
        barcodeCameraConfiguration.setBarcodeFormatsFilter(nativeBarcodeFormats)
    }
    if (arguments.containsKey("acceptedDocumentFormats")) {
        val acceptedDocumentFormats = arguments["acceptedDocumentFormats"] as ArrayList<String>
        val acceptedBarcodeDocumentFormat = ArrayList<BarcodeDocumentFormat>()
        for (formatValue in acceptedDocumentFormats) {
            acceptedBarcodeDocumentFormat.add(
                BarcodeDocumentFormat.valueOf(
                    formatValue.replaceFirst(
                        "AcceptedDocumentFormat.",
                        ""
                    )
                )
            )
        }
        barcodeCameraConfiguration.setAcceptedDocumentFormats(acceptedBarcodeDocumentFormat)
    }
    if (arguments.containsKey("barcodeImageGenerationType")) {
        val imageGenType =
            BarcodeImageGenerationType.valueOf(arguments["barcodeImageGenerationType"] as String)
        barcodeCameraConfiguration.setBarcodeImageGenerationType(imageGenType)
    }
    if (arguments.containsKey("replaceCancelButtonWithIcon")) {
        if (arguments["replaceCancelButtonWithIcon"] == true){
            barcodeCameraConfiguration.setCancelButtonIcon(R.drawable.ic_baseline_arrow_back_24);
        }
    }
    return barcodeCameraConfiguration
}

private fun checkFinderAspectRatioParam(
    call: MethodCall,
    arguments: BarcodeScannerConfigurationJson
): FinderAspectRatio? {
    val propertyName = "finderAspectRatio"
    if (arguments[propertyName] != null) {
        val aspRatio = arguments[propertyName] as? AspectRatioJson
        val width = aspRatio?.get("width") as? Int
        val height = aspRatio?.get("height") as? Int
        arguments.remove(propertyName)
        return FinderAspectRatio(width?.toDouble() ?: 1.0, height?.toDouble() ?: 1.0)
    }
    return null
}

private fun getAdditionalConfig(it: Json): BarcodeScannerAdditionalConfiguration {
    val defaultConfig = BarcodeScannerAdditionalConfiguration()

    val minimumTextLength = it["minimumTextLength"] as? Int
        ?: defaultConfig.minimumTextLength
    val gs1DecodingEnabled = it["enableGS1Decoding"] as? Boolean
        ?: defaultConfig.gs1DecodingEnabled
    val maximumTextLength = it["maximumTextLength"] as? Int
        ?: defaultConfig.maximumTextLength
    val minimum1DBarcodesQuietZone = it["minimum1DBarcodesQuietZone"] as? Int
        ?: defaultConfig.minimum1DQuietZoneSize
    val stripCheckDigits = it["stripCheckDigits"] as? Boolean
        ?: defaultConfig.stripCheckDigits
    val lowPowerMode = it["lowPowerMode"] as? Boolean
        ?: defaultConfig.lowPowerMode
    val codeDensity = extractBarcodeDensityFrom(it["codeDensity"] as? String)
        ?: defaultConfig.codeDensity

    var msiPlesseyChecksumAlgorithm: EnumSet<MSIPlesseyChecksumAlgorithm> =
        when (it["msiPlesseyChecksumAlgorithm"]) {
            "None" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.None)
            "Mod10" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod10)
            "Mod11IBM" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod11IBM)
            "Mod11NCR" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod11NCR)
            "Mod1010" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod1010)
            "Mod1110IBM" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod1110IBM)
            "Mod1110NCR" -> EnumSet.of(MSIPlesseyChecksumAlgorithm.Mod1110NCR)
            else -> defaultConfig.msiPlesseyChecksumAlgorithms
        }

    return BarcodeScannerAdditionalConfiguration(
        gs1DecodingEnabled = gs1DecodingEnabled,
        minimumTextLength = minimumTextLength,
        maximumTextLength = maximumTextLength,
        minimum1DQuietZoneSize = minimum1DBarcodesQuietZone,
        stripCheckDigits = stripCheckDigits,
        msiPlesseyChecksumAlgorithms = msiPlesseyChecksumAlgorithm,
        lowPowerMode = lowPowerMode,
        codeDensity = codeDensity
    )
}

private fun checkCameraOrientationParam(
    call: MethodCall,
    arguments: BarcodeScannerConfigurationJson
): CameraOrientationMode? {
    if (call.hasArgument("orientationLockMode")) {
        val value = when (call.argument<String>("orientationLockMode")) {
            "PORTRAIT" -> CameraOrientationMode.PORTRAIT
            "LANDSCAPE" -> CameraOrientationMode.LANDSCAPE
            else -> null
        }
        arguments.remove("orientationLockMode")
        return value
    }
    return null
}

/**
 * Extract the barcode density value from enum.
 */
internal fun extractBarcodeDensityFrom(string: String?) : BarcodeDensity? {
    when(string ?: "") {
        "LOW" -> return BarcodeDensity.LOW
        "HIGH" -> return BarcodeDensity.HIGH
    }
    return null
}

/**
 * Extract the barcode density value from enum.
 */
internal fun extractEngineModeFrom(engineMode: String?) : EngineMode {
    when(engineMode ?: "") {
        "Legacy" -> return EngineMode.LegacyFast
        "NextGen" -> return EngineMode.NextGen
    }
    return EngineMode.NextGen
}

